package java.time.chrono;

import java.time.temporal.Temporal;
import java.io.DataInput;
import java.io.IOException;
import java.io.DataOutput;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.time.Period;
import java.time.LocalTime;
import java.time.temporal.TemporalUnit;
import java.time.temporal.TemporalAmount;
import java.time.temporal.TemporalAdjuster;
import java.time.temporal.UnsupportedTemporalTypeException;
import java.time.temporal.ChronoField;
import java.time.temporal.ValueRange;
import java.time.temporal.TemporalField;
import java.util.Objects;
import java.time.temporal.TemporalAccessor;
import java.time.ZoneId;
import java.time.Clock;
import java.time.LocalDate;
import java.io.Serializable;

public final class ThaiBuddhistDate extends ChronoLocalDateImpl<ThaiBuddhistDate> implements ChronoLocalDate, Serializable
{
    private static final long serialVersionUID = -8722293800195731463L;
    private final transient LocalDate isoDate;
    
    public static ThaiBuddhistDate now() {
        return now(Clock.systemDefaultZone());
    }
    
    public static ThaiBuddhistDate now(ZoneId zone) {
        return now(Clock.system(zone));
    }
    
    public static ThaiBuddhistDate now(Clock clock) {
        return new ThaiBuddhistDate(LocalDate.now(clock));
    }
    
    public static ThaiBuddhistDate of(int n, int month, int dayOfMonth) {
        return new ThaiBuddhistDate(LocalDate.of(n - 543, month, dayOfMonth));
    }
    
    public static ThaiBuddhistDate from(TemporalAccessor temporalAccessor) {
        return ThaiBuddhistChronology.INSTANCE.date(temporalAccessor);
    }
    
    ThaiBuddhistDate(LocalDate localDate) {
        Objects.requireNonNull(localDate, "isoDate");
        this.isoDate = localDate;
    }
    
    @Override
    public ThaiBuddhistChronology getChronology() {
        return ThaiBuddhistChronology.INSTANCE;
    }
    
    @Override
    public ThaiBuddhistEra getEra() {
        return (this.getProlepticYear() >= 1) ? ThaiBuddhistEra.BE : ThaiBuddhistEra.BEFORE_BE;
    }
    
    @Override
    public int lengthOfMonth() {
        return this.isoDate.lengthOfMonth();
    }
    
    @Override
    public ValueRange range(TemporalField temporalField) {
        if ((temporalField instanceof ChronoField)) {
            if (this.isSupported(temporalField)) {
                ChronoField chronoField = (ChronoField)temporalField;
                switch (chronoField) {
                    case DAY_OF_MONTH:
                    case DAY_OF_YEAR:
                    case ALIGNED_WEEK_OF_MONTH: {
                        return this.isoDate.range(temporalField);
                    }
                    case YEAR_OF_ERA: {
                        ValueRange range = ChronoField.YEAR.range();
                        return ValueRange.of(1L, (this.getProlepticYear() <= 0) ? (-(range.getMinimum() + 543L) + 1L) : (range.getMaximum() + 543L));
                    }
                    default: {
                        return this.getChronology().range(chronoField);
                    }
                }
            }
            else {
                throw new UnsupportedTemporalTypeException("Unsupported field: " + temporalField);
            }
        }
        else {
            return temporalField.rangeRefinedBy(this);
        }
    }
    
    @Override
    public long getLong(TemporalField field) {
        if ((field instanceof ChronoField)) {
            switch ((ChronoField)field) {
                case PROLEPTIC_MONTH: {
                    return this.getProlepticMonth();
                }
                case YEAR_OF_ERA: {
                    int prolepticYear = this.getProlepticYear();
                    return (long)((prolepticYear >= 1) ? prolepticYear : (1 - prolepticYear));
                }
                case YEAR: {
                    return (long)this.getProlepticYear();
                }
                case ERA: {
                    return (long)(this.getProlepticYear() >= 1);
                }
                default: {
                    return this.isoDate.getLong(field);
                }
            }
        }
        else {
            return field.getFrom(this);
        }
    }
    
    private long getProlepticMonth() {
        return (long)this.getProlepticYear() * 12L + (long)this.isoDate.getMonthValue() - 1L;
    }
    
    private int getProlepticYear() {
        return this.isoDate.getYear() + 543;
    }
    
    @Override
    public ThaiBuddhistDate with(TemporalField field, long newValue) {
        if ((field instanceof ChronoField)) {
            ChronoField chronoField = (ChronoField)field;
            if (this.getLong(chronoField) == newValue) {
                return this;
            }
            switch (chronoField) {
                case PROLEPTIC_MONTH: {
                    this.getChronology().range(chronoField).checkValidValue(newValue, chronoField);
                    return this.plusMonths(newValue - this.getProlepticMonth());
                }
                case YEAR_OF_ERA:
                case YEAR:
                case ERA: {
                    int checkValidIntValue = this.getChronology().range(chronoField).checkValidIntValue(newValue, chronoField);
                    switch (chronoField) {
                        case YEAR_OF_ERA: {
                            return this.with(this.isoDate.withYear(((this.getProlepticYear() >= 1) ? checkValidIntValue : (1 - checkValidIntValue)) - 543));
                        }
                        case YEAR: {
                            return this.with(this.isoDate.withYear(checkValidIntValue - 543));
                        }
                        case ERA: {
                            return this.with(this.isoDate.withYear(1 - this.getProlepticYear() - 543));
                        }
                        default: {
                            break;
                        }
                    }
                    break;
                }
            }
            return this.with(this.isoDate.with(field, newValue));
        }
        else {
            return (ThaiBuddhistDate)super.with(field, newValue);
        }
    }
    
    @Override
    public ThaiBuddhistDate with(TemporalAdjuster temporalAdjuster) {
        return (ThaiBuddhistDate)super.with(temporalAdjuster);
    }
    
    @Override
    public ThaiBuddhistDate plus(TemporalAmount temporalAmount) {
        return (ThaiBuddhistDate)super.plus(temporalAmount);
    }
    
    @Override
    public ThaiBuddhistDate minus(TemporalAmount temporalAmount) {
        return (ThaiBuddhistDate)super.minus(temporalAmount);
    }
    
    @Override
    ThaiBuddhistDate plusYears(long yearsToAdd) {
        return this.with(this.isoDate.plusYears(yearsToAdd));
    }
    
    @Override
    ThaiBuddhistDate plusMonths(long monthsToAdd) {
        return this.with(this.isoDate.plusMonths(monthsToAdd));
    }
    
    @Override
    ThaiBuddhistDate plusWeeks(long n) {
        return (ThaiBuddhistDate)super.plusWeeks(n);
    }
    
    @Override
    ThaiBuddhistDate plusDays(long daysToAdd) {
        return this.with(this.isoDate.plusDays(daysToAdd));
    }
    
    @Override
    public ThaiBuddhistDate plus(long n, TemporalUnit temporalUnit) {
        return (ThaiBuddhistDate)super.plus(n, temporalUnit);
    }
    
    @Override
    public ThaiBuddhistDate minus(long n, TemporalUnit temporalUnit) {
        return (ThaiBuddhistDate)super.minus(n, temporalUnit);
    }
    
    @Override
    ThaiBuddhistDate minusYears(long n) {
        return (ThaiBuddhistDate)super.minusYears(n);
    }
    
    @Override
    ThaiBuddhistDate minusMonths(long n) {
        return (ThaiBuddhistDate)super.minusMonths(n);
    }
    
    @Override
    ThaiBuddhistDate minusWeeks(long n) {
        return (ThaiBuddhistDate)super.minusWeeks(n);
    }
    
    @Override
    ThaiBuddhistDate minusDays(long n) {
        return (ThaiBuddhistDate)super.minusDays(n);
    }
    
    private ThaiBuddhistDate with(LocalDate localDate) {
        return localDate.equals(this.isoDate) ? this : new ThaiBuddhistDate(localDate);
    }
    
    @Override
    public final ChronoLocalDateTime<ThaiBuddhistDate> atTime(LocalTime localTime) {
        return super.atTime(localTime);
    }
    
    @Override
    public ChronoPeriod until(ChronoLocalDate endDateExclusive) {
        Period until = this.isoDate.until(endDateExclusive);
        return this.getChronology().period(until.getYears(), until.getMonths(), until.getDays());
    }
    
    @Override
    public long toEpochDay() {
        return this.isoDate.toEpochDay();
    }
    
    @Override
    public boolean equals(Object o) {
        return this == o || (o instanceof ThaiBuddhistDate && this.isoDate.equals(((ThaiBuddhistDate)o).isoDate));
    }
    
    @Override
    public int hashCode() {
        return this.getChronology().getId().hashCode() ^ this.isoDate.hashCode();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws InvalidObjectException {
        throw new InvalidObjectException("Deserialization via serialization delegate");
    }
    
    private Object writeReplace() {
        return new Ser((byte)8, this);
    }
    
    void writeExternal(DataOutput dataOutput) throws IOException {
        dataOutput.writeInt(this.get(ChronoField.YEAR));
        dataOutput.writeByte(this.get(ChronoField.MONTH_OF_YEAR));
        dataOutput.writeByte(this.get(ChronoField.DAY_OF_MONTH));
    }
    
    static ThaiBuddhistDate readExternal(DataInput dataInput) throws IOException {
        return ThaiBuddhistChronology.INSTANCE.date(dataInput.readInt(), (int)dataInput.readByte(), (int)dataInput.readByte());
    }
}
